//********************************************************************
UNIT GUIDescendants;
//********************************************************************
INTERFACE

USES GUIItem, GUIBase, GraphicSupport;
//********************************************************************
TYPE
   TShapeType = (stLine, stRectangle, stRoundRect, stEllipse);
//********************************************************************
   TGUIShape = Class(TGUIItem)
   Private
      FDown: Boolean;
      FShapeType: TShapeType;
   Public
      Constructor Create(AName: String; AParent: TGUIItem;
                      AShapeType: TShapeType); Reintroduce;

      Procedure OnMouseBtnLDown
         (AMouseBtnEventData: TMouseBtnEventData); Override;
      Procedure OnMouseBtnLUp
         (AMouseBtnEventData: TMouseBtnEventData); Override;

      Procedure DrawOwnData; Override;
   End;
//********************************************************************
IMPLEMENTATION
//********************************************************************
//********************************************************************
Constructor TGUIShape.Create(AName: String; AParent: TGUIItem;
                         AShapeType: TShapeType);
Begin
   Inherited Create(AName,AParent);
   FShapeType:= AShapeType;
   FDown:= False;
End;
//********************************************************************
Procedure TGUIShape.OnMouseBtnLDown(AMouseBtnEventData: TMouseBtnEventData);
Begin
   FDown:= True;
   Inherited OnMouseBtnLDown(AMouseBtnEventData);
End;
//********************************************************************
Procedure TGUIShape.OnMouseBtnLUp(AMouseBtnEventData: TMouseBtnEventData);
Begin
   FDown:= False;
   Inherited OnMouseBtnLUp(AMouseBtnEventData);
End;
//********************************************************************
Procedure TGUIShape.DrawOwnData;
Begin
   Inherited DrawOwnData;

   With Themes Do If ActivateColor(FGColorIndex)
      Then Begin
         If FDown
            Then Themes.Color.SetLuminance(0.5);
         Case FShapeType Of
            stLine: Line(Left,Top,Right,Bottom);
            stRectangle: Rectangle(Left,Top,Right,Bottom, True);
            stRoundRect: RoundRect(Left,Top,Right,Bottom, True);
            stEllipse: Ellipse(Left,Top,Right,Bottom, True);
         End;
         If FDown
            Then Themes.Color.Reset;
      End;
End;
//********************************************************************
//********************************************************************
End.
