//********************************************************************
UNIT GUIItem;
// This unit contains the TGUIItem, which is the ancestor of all
// TGUI-Descendents. A bunch of TGUIItems can bild up a tree that
// can be used as bidirectional data delivering net to
//    * every one of the participating TGUIItems
//    * a user pascal program
//    * a script
// implements methods that are equal for all its descendants,
// e.g. Load/Save, default event handlers, Hit, etc.
//********************************************************************
INTERFACE

USES Contnrs, GraphicSupport, GUIBase;
//********************************************************************
TYPE
//********************************************************************
// User event procedure handlers
   TKeyEventProc = Procedure(AKeyEventData: TKeyEventData);
   TMouseBtnEventProc = Procedure(AMouseBtnEventData: TMouseBtnEventData);
   TMouseMoveEventProc = Procedure(AMouseMoveEventData: TMouseMoveEventData);
   TMouseWheelEventProc = Procedure(AMouseWheelEventData: TMouseWheelEventData);
//********************************************************************
// User event method handlers
   TKeyEventMethod = Procedure(AKeyEventData: TKeyEventData) Of Object;
   TMouseBtnEventMethod = Procedure(AMouseBtnEventData: TMouseBtnEventData) Of Object;
   TMouseMoveEventMethod = Procedure(AMouseMoveEventData: TMouseMoveEventData) Of Object;
   TMouseWheelEventMethod = Procedure(AMouseWheelEventData: TMouseWheelEventData) Of Object;
//********************************************************************
   TGUIItem = Class;
   TNoticeBoard = Record
      FocusedItem: TGUIItem;
   End;
//********************************************************************
{$M+}
   TGUIItem = Class(TObject)
   Private
      // General data
      FName: TGUIIdentity;

      FFocus: Boolean;
      FVisible: TGUIVisible;
      FBGColorIndex: TGUIThemesIndex;
      FBGColorName: TGUIColorName;
      FFGColorIndex: TGUIThemesIndex;
      FFGColorName: TGUIColorName;
      FLeft,FTop,FRight,FBottom: TGUIPosition;

      FParent: TGUIItem;      // only a pointer, do not destroy!
      FChildren: TObjectList;

      FDataProvider: TGUIDataProvider; // only a pointer, do not destroy!
      FThemes: TThemes;  // only a pointer, do not destroy!

      // User's event-procedures
      FProcKeyDown: TKeyEventProc;
      FProcKeyUp: TKeyEventProc;
      FProcMouseBtnLDown: TMouseBtnEventProc;
      FProcMouseBtnLUp: TMouseBtnEventProc;
      FProcMouseBtnRDown: TMouseBtnEventProc;
      FProcMouseBtnMDown: TMouseBtnEventProc;
      FProcMouseMove: TMouseMoveEventProc;
      FProcMouseWheelDown: TMouseWheelEventProc;
      FProcMouseWheelUp: TMouseWheelEventProc;

      // User's event-methods
      FMethodKeyDown: TKeyEventMethod;
      FMethodKeyUp: TKeyEventMethod;
      FMethodMouseBtnLDown: TMouseBtnEventMethod;
      FMethodMouseBtnLUp: TMouseBtnEventMethod;
      FMethodMouseBtnRDown: TMouseBtnEventMethod;
      FMethodMouseBtnMDown: TMouseBtnEventMethod;
      FMethodMouseMove: TMouseMoveEventMethod;
      FMethodMouseWheelDown: TMouseWheelEventMethod;
      FMethodMouseWheelUp: TMouseWheelEventMethod;

      // Scripts's eventhandlers
      FScriptKeyDown: String;
      FScriptKeyUp: String;
      FScriptMouseBtnLDown: String;
      FScriptMouseBtnLUp: String;
      FScriptMouseBtnRDown: String;
      FScriptMouseBtnMDown: String;
      FScriptMouseMove: String;
      FScriptMouseWheelDown: String;
      FScriptMouseWheelUp: String;

      Procedure SetBGColorName(AColorName: TGUIColorName);
      Procedure SetFGColorName(AColorName: TGUIColorName);
   Protected
      // Virtual methods
      Procedure DrawOwnData; Virtual;
       Function Hit2DOwnData(X,Y: Integer): Boolean; Virtual;
      Procedure Scale2DOwnData(X,Y: Integer); Virtual;
      Procedure Shift2DOwnData(X,Y: Integer); Virtual;
      Procedure LoadOwnData; Virtual;
      Procedure SaveOwnData; Virtual;
   Public
      // User's event-procedures as properties
      Property ProcKeyDown: TKeyEventProc
         Write FProcKeyDown;
      Property ProcKeyUp: TKeyEventProc
         Write FProcKeyUp;
      Property ProcMouseBtnLDown: TMouseBtnEventProc
         Write FProcMouseBtnLDown;
      Property ProcMouseBtnRDown: TMouseBtnEventProc
         Write FProcMouseBtnRDown;
      Property ProcMouseBtnMDown: TMouseBtnEventProc
         Write FProcMouseBtnMDown;
      Property ProcMouseMove: TMouseMoveEventProc
         Write FProcMouseMove;
      Property ProcMouseWheelDown: TMouseWheelEventProc
         Write FProcMouseWheelDown;
      Property ProcMouseWheelUp: TMouseWheelEventProc
         Write FProcMouseWheelUp;

      // User's event-methods as properties
      Property MethodKeyDown: TKeyEventMethod
         Write FMethodKeyDown;
      Property MethodKeyUp: TKeyEventMethod
         Write FMethodKeyUp;
      Property MethodMouseBtnLDown: TMouseBtnEventMethod
         Write FMethodMouseBtnLDown;
      Property MethodMouseBtnRDown: TMouseBtnEventMethod
         Write FMethodMouseBtnRDown;
      Property MethodMouseBtnMDown: TMouseBtnEventMethod
         Write FMethodMouseBtnMDown;
      Property MethodMouseMove: TMouseMoveEventMethod
         Write FMethodMouseMove;
      Property MethodMouseWheelDown: TMouseWheelEventMethod
         Write FMethodMouseWheelDown;
      Property MethodMouseWheelUp: TMouseWheelEventMethod
         Write FMethodMouseWheelUp;

      // Scripts's eventhandlers as properties
      Property ScriptKeyDown: String
         Read FScriptKeyDown Write FScriptKeyDown;
      Property ScriptKeyUp: String
         Read FScriptKeyUp Write FScriptKeyUp;
      Property ScriptMouseBtnLDown: String
         Read FScriptMouseBtnLDown Write FScriptMouseBtnLDown;
      Property ScriptMouseBtnLUp: String
         Read FScriptMouseBtnLUp Write FScriptMouseBtnLUp;
      Property ScriptMouseBtnRDown: String
         Read FScriptMouseBtnRDown Write FScriptMouseBtnRDown;
      Property ScriptMouseBtnMDown: String
         Read FScriptMouseBtnMDown Write FScriptMouseBtnMDown;
      Property ScriptMouseMove: String
         Read FScriptMouseMove Write FScriptMouseMove;
      Property ScriptMouseWheelDown: String
         Read FScriptMouseWheelDown Write FScriptMouseWheelDown;
      Property ScriptMouseWheelUp: String
         Read FScriptMouseWheelUp Write FScriptMouseWheelUp;

      // Other properties
      Property BGColorIndex: TGUIThemesIndex Read FBGColorIndex;
      Property FGColorIndex: TGUIThemesIndex Read FFGColorIndex;

      Property Themes: TThemes Read FThemes;
      Property DataProvider: TGUIDataProvider
         Read FDataProvider Write FDataProvider;

      // Constructor/Destructor/Registration -------------------------
      Constructor Create(AName: String; AParent: TGUIItem);
                                   Reintroduce; Virtual;
      Destructor Destroy; Override; // TREE-Method
      Procedure Register(AGUIItem: TGUIItem);
      Procedure UnRegister(AGUIItem: TGUIItem);
      Procedure SetParent(AParent: TGUIItem);

      // Events: TREE-Methods, virtual -------------------------------
      Procedure OnKeyDown(AKeyEventData: TKeyEventData); Virtual;
      Procedure OnKeyUp(AKeyEventData: TKeyEventData); Virtual;
      Procedure OnMouseBtnLDown(AMouseBtnEventData: TMouseBtnEventData); Virtual;
      Procedure OnMouseBtnLUp(AMouseBtnEventData: TMouseBtnEventData); Virtual;
      Procedure OnMouseBtnRDown(AMouseBtnEventData: TMouseBtnEventData); Virtual;
      Procedure OnMouseBtnMDown(AMouseBtnEventData: TMouseBtnEventData); Virtual;
      Procedure OnMouseMove(AMouseMoveEventData: TMouseMoveEventData); Virtual;
      Procedure OnMouseWheelDown(AMouseWheelEventData: TMouseWheelEventData); Virtual;
      Procedure OnMouseWheelUp(AMouseWheelEventData: TMouseWheelEventData); Virtual;

      // Other TREE-Methods (static) ---------------------------------
      Procedure Draw;
      Function Hit2D(Var ANoticeBoard: TNoticeBoard; X,Y: Integer): Boolean;
      Procedure Scale2D(X,Y: Integer);
      Procedure Shift2D(X,Y: Integer);
      Procedure Load;
      Procedure Save;
   Published
      // General properties
      Property Focus: Boolean Read FFocus Write FFocus;
      Property Name: String Read FName; // Name is read-only
      Property Visible: Boolean Read FVisible Write FVisible;

      Property BGColorName: TGUIColorName
         Read FBGColorName Write SetBGColorName;
      Property FGColorName: TGUIColorName
         Read FFGColorName Write SetFGColorName;

      // Position properties
      Property Left: TGUIPosition Read FLeft Write FLeft;
      Property Top: TGUIPosition Read FTop Write FTop;
      Property Right: TGUIPosition Read FRight Write FRight;
      Property Bottom: TGUIPosition Read FBottom Write FBottom;

   End;
{$M+}
//********************************************************************
   TGUIManager = Class(TGUIItem)
   Private
      FGUIQuit: Boolean;
      FNoticeBoard: TNoticeBoard;
      GRS_Environment: TGRS_Environment;

      Procedure SetDataProvider(ADataProvider: TGUIDataProvider);
   Public
      Property DataProvider Write SetDataProvider;
      Property GUIQuit: Boolean Read FGUIQuit;

      Constructor Create(AName: String; AParent: TGUIItem); Override;
      Destructor Destroy; Override;
       Procedure CatchEvent;
       Procedure DrawGUI;
       Procedure GetFocusedItemOnMouseClick
                 (AMouseBtnEventData: TMouseBtnEventData);
        Function InitGraphicSupport: Boolean;
   End;
//********************************************************************
IMPLEMENTATION

//Var Lua: Lua_State;
//********************************************************************
//   T G U I I T E M   -   Constructor/Destructor/Registration
//********************************************************************
// Initialization
Constructor TGUIItem.Create(AName: String; AParent: TGUIItem);
Begin
   Inherited Create;

   FName:= AName;
   FParent:= Nil;
   SetParent(AParent);
   FChildren:= TObjectList.Create;

   FFocus:= False;

   FVisible:= True;
   FLeft:=0; FTop:=0; FRight:=0; FBottom:=0;

   If FParent <> Nil Then Begin
      FDataProvider:= FParent.DataProvider;

      FThemes:= FParent.Themes;
      FBGColorIndex:= FThemes.ColorIndex('SolidGrey');
      FFGColorIndex:= FThemes.ColorIndex('SolidOrange');
   End;
End;
//********************************************************************
// TREE-method: Destroys all children of the objectlist
Destructor TGUIItem.Destroy;
Begin
   FChildren.Free; // TObjectList frees all children
   Inherited Destroy;
End;
//********************************************************************
// Adds a GUIItem to the list
Procedure TGUIItem.Register(AGUIItem: TGUIItem);
Begin
   FChildren.Add(AGUIItem);
End;
//********************************************************************
// Set a new parent
Procedure TGUIItem.SetParent(AParent: TGUIItem);
Begin
   If Assigned(FParent)
      Then FParent.Unregister(Self);
   FParent:= AParent;
   If Assigned(FParent)
      Then FParent.Register(Self);
End;
//********************************************************************
// Deletes a GUIItem in the list
Procedure TGUIItem.Unregister(AGUIItem: TGUIItem);
Var Index: Integer;
Begin
   With FChildren Do Begin
      Index:= IndexOf(AGUIItem);
      If Index > -1 Then Delete(Index);
   End;
End;
//********************************************************************
//   T G U I I T E M   -   Events: TREE-Methods, virtual
//********************************************************************
// TREE-method: Receives a keydown-event and and sends it to
//  * Calling program (procedure/method)
//  * Calling script
//  * Calling Children
Procedure TGUIItem.OnKeyDown(AKeyEventData: TKeyEventData);
Var LocalChild: TGUIItem; I: Integer;
Begin
   // This eventhandler is to be overridden by GUIDescendents
   // The overriding method must call this eventhandler at its end

   // User's procedure-eventhandler
   If Assigned(FProcKeyDown)
      Then FProcKeyDown(AKeyEventData);

   // User's method-eventhandler
   If Assigned(FMethodKeyDown)
      Then FMethodKeyDown(AKeyEventData);

   // Script's eventhandler &&&&&
   {If ScriptKeyDown <> '' Then With AKeyEventData Do Begin
      Lua_GetGlobal(Lua, PChar(ScriptKeyDown));
      Lua_PushNumber(Lua, Key);
      With AdditionalKeys Do Begin
         Lua_PushBoolean(Lua, ssNone);
         Lua_PushBoolean(Lua, ssShift);
         Lua_PushBoolean(Lua, ssAlt);
         Lua_PushBoolean(Lua, ssCtrl);
      End;
      Lua_Call(Lua,9,0); // Lua_PCall? &&&&&
   End; }

   With FChildren Do If Count > 0      // Dispatch to children
      Then For I:=0 To Count-1 Do Begin
         LocalChild:= List[I];
         LocalChild.OnKeyDown(AKeyEventData);
      End;
End;
//********************************************************************
// Comment: see method "OnKeyDown"
Procedure TGUIItem.OnKeyUp(AKeyEventData: TKeyEventData);
Var LocalChild: TGUIItem; I: Integer;
Begin
   // User's procedure-eventhandler
   If Assigned(FProcKeyUp)
      Then FProcKeyUp(AKeyEventData);

   // User's method-eventhandler
   If Assigned(FMethodKeyUp)
      Then FMethodKeyUp(AKeyEventData);

   With FChildren Do If Count > 0      // Dispatch to children
      Then For I:=0 To Count-1 Do Begin
         LocalChild:= List[I];
         LocalChild.OnKeyUp(AKeyEventData);
      End;
End;
//********************************************************************
// Comment: see method "OnKeyDown"
Procedure TGUIItem.OnMouseBtnLDown(AMouseBtnEventData: TMouseBtnEventData);
Var LocalChild: TGUIItem; I: Integer;
Begin
   // User's procedure-eventhandler
   If Assigned(FProcMouseBtnLDown)
      Then FProcMouseBtnLDown(AMouseBtnEventData);

   // User's method-eventhandler
   If Assigned(FMethodMouseBtnLDown)
      Then FMethodMouseBtnLDown(AMouseBtnEventData);

   {With FChildren Do If Count > 0      // Dispatch to children
      Then For I:=0 To Count-1 Do Begin
         LocalChild:= List[I];
         LocalChild.OnMouseBtnLDown(AMouseBtnEventData);
      End;}
End;
//********************************************************************
// Comment: see method "OnKeyDown"
Procedure TGUIItem.OnMouseBtnLUp(AMouseBtnEventData: TMouseBtnEventData);
Var LocalChild: TGUIItem; I: Integer;
Begin
   // User's procedure-eventhandler
   If Assigned(FProcMouseBtnLUp)
      Then FProcMouseBtnLUp(AMouseBtnEventData);

   // User's method-eventhandler
   If Assigned(FMethodMouseBtnLUp)
      Then FMethodMouseBtnLUp(AMouseBtnEventData);

   {With FChildren Do If Count > 0      // Dispatch to children
      Then For I:=0 To Count-1 Do Begin
         LocalChild:= List[I];
         LocalChild.OnMouseBtnLUp(AMouseBtnEventData);
      End; }
End;
//********************************************************************
// Comment: see method "OnKeyDown"
Procedure TGUIItem.OnMouseBtnRDown(AMouseBtnEventData: TMouseBtnEventData);
Var LocalChild: TGUIItem; I: Integer;
Begin
   // User's procedure-eventhandler
   If Assigned(FProcMouseBtnRDown)
      Then FProcMouseBtnRDown(AMouseBtnEventData);

   // User's method-eventhandler
   If Assigned(FMethodMouseBtnRDown)
      Then FMethodMouseBtnRDown(AMouseBtnEventData);

   {With FChildren Do If Count > 0      // Dispatch to children
      Then For I:=0 To Count-1 Do Begin
         LocalChild:= List[I];
         LocalChild.OnMouseBtnRDown(AMouseBtnEventData);
      End; }
End;
//********************************************************************
// Comment: see method "OnKeyDown"
Procedure TGUIItem.OnMouseBtnMDown(AMouseBtnEventData: TMouseBtnEventData);
Var LocalChild: TGUIItem; I: Integer;
Begin
   // User's procedure-eventhandler
   If Assigned(FProcMouseBtnMDown)
      Then FProcMouseBtnMDown(AMouseBtnEventData);

   // User's method-eventhandler
   If Assigned(FMethodMouseBtnMDown)
      Then FMethodMouseBtnMDown(AMouseBtnEventData);

   {With FChildren Do If Count > 0      // Dispatch to children
      Then For I:=0 To Count-1 Do Begin
         LocalChild:= List[I];
         LocalChild.OnMouseBtnMDown(AMouseBtnEventData);
      End; }
End;
//********************************************************************
// Comment: see method "OnKeyDown"
Procedure TGUIItem.OnMouseMove(AMouseMoveEventData: TMouseMoveEventData);
Var LocalChild: TGUIItem; I: Integer;
Begin
   // User's procedure-eventhandler
   If Assigned(FProcMouseMove)
      Then FProcMouseMove(AMouseMoveEventData);

   // User's method-eventhandler
   If Assigned(FMethodMouseMove)
      Then FMethodMouseMove(AMouseMoveEventData);

   {With FChildren Do If Count > 0      // Dispatch to children
      Then For I:=0 To Count-1 Do Begin
         LocalChild:= List[I];
         LocalChild.OnMouseMove(AMouseMoveEventData);
      End; }
End;
//********************************************************************
// Comment: see method "OnKeyDown"
Procedure TGUIItem.OnMouseWheelDown(AMouseWheelEventData: TMouseWheelEventData);
Var LocalChild: TGUIItem; I: Integer;
Begin
   // User's procedure-eventhandler
   If Assigned(FProcMouseWheelDown)
      Then FProcMouseWheelDown(AMouseWheelEventData);

   // User's method-eventhandler
   If Assigned(FMethodMouseWheelDown)
      Then FMethodMouseWheelDown(AMouseWheelEventData);

  { With FChildren Do If Count > 0      // Dispatch to children
      Then For I:=0 To Count-1 Do Begin
         LocalChild:= List[I];
         LocalChild.OnMouseWheelDown(AMouseWheelEventData);
      End; }
End;
//********************************************************************
// Comment: see method "OnKeyDown"
Procedure TGUIItem.OnMouseWheelUp(AMouseWheelEventData: TMouseWheelEventData);
Var LocalChild: TGUIItem; I: Integer;
Begin
   // User's procedure-eventhandler
   If Assigned(FProcMouseWheelUp)
      Then FProcMouseWheelUp(AMouseWheelEventData);

   // User's method-eventhandler
   If Assigned(FMethodMouseWheelUp)
      Then FMethodMouseWheelUp(AMouseWheelEventData);

   {With FChildren Do If Count > 0      // Dispatch to children
      Then For I:=0 To Count-1 Do Begin
         LocalChild:= List[I];
         LocalChild.OnMouseWheelUp(AMouseWheelEventData);
      End; }
End;
//********************************************************************
//   T G U I I T E M   -   Other methods
//********************************************************************
// TREE-method: forces all children to draw
Procedure TGUIItem.Draw;
Var LocalChild: TGUIItem; I: Integer;
Begin
   DrawOwnData;

   With FChildren Do If Count > 0
      Then For I:=0 To Count-1 Do Begin
         LocalChild:= List[I];
         LocalChild.Draw;
      End;
End;
//********************************************************************
// Virtual method - descendents should overwrite
// Draws the background rectangle
Procedure TGUIItem.DrawOwnData;
Begin
   With Themes Do If ActivateColor(FBGColorIndex)
      Then Rectangle(FLeft,FTop,FRight,FBottom, True);
End;
//********************************************************************
// TREE-method: forces all children to check if hit
// Hit TGUIItems register themselves at the NoticeBoard
Function TGUIItem.Hit2D(Var ANoticeBoard: TNoticeBoard;
                                  X,Y: Integer): Boolean;
Var LocalChild: TGUIItem; I: Integer; ItemHit: Boolean;
Begin
   Result:= False;
   If Hit2DOwnData(X,Y) Then Begin
      Result:= True;
      ANoticeBoard.FocusedItem:= Self;

      With FChildren Do If Count > 0 Then Begin
         I:=0; ItemHit:= False;
         While (I < Count) And (Not ItemHit) Do Begin
            LocalChild:= List[I];
            ItemHit:= LocalChild.Hit2D(ANoticeBoard, X,Y);
            If Not ItemHit Then Inc(I); //================>
         End;
      End;
   End;
End;
//********************************************************************
// Virtual method - descendents should overwrite
Function TGUIItem.Hit2DOwnData(X,Y: Integer): Boolean;
Begin
   Result:= False;
   If (X >= Left) And (X <= Right) And
      (Y >= Top) And (Y <= Bottom)
      Then Result:= True;
End;
//********************************************************************
// TREE-method: forces all children to scale
Procedure TGUIItem.Scale2D(X,Y: Integer);
Var LocalChild: TGUIItem; I: Integer;
Begin
   Scale2DOwnData(X,Y);

   With FChildren Do If Count > 0
      Then For I:=0 To Count-1 Do Begin
         LocalChild:= List[I];
         LocalChild.Scale2D(X,Y);
      End;
End;
//********************************************************************
// Virtual method - descendents should overwrite
Procedure TGUIItem.Scale2DOwnData(X,Y: Integer);
Begin
   FRight:= FLeft + X;
   FBottom:= FTop + Y;
End;
//********************************************************************
// TREE-method: forces all children to shift
Procedure TGUIItem.Shift2D(X,Y: Integer);
Var LocalChild: TGUIItem; I: Integer;
Begin
   Shift2DOwnData(X,Y);

   With FChildren Do If Count > 0
      Then For I:=0 To Count-1 Do Begin
         LocalChild:= List[I];
         LocalChild.Shift2D(X,Y);
      End;
End;
//********************************************************************
// Virtual method - descendents should overwrite
Procedure TGUIItem.Shift2DOwnData(X,Y: Integer);
Begin
   Inc(FLeft,X);
   Inc(FRight,X);
   Inc(FTop,Y);
   Inc(FBottom,Y);
End;
//********************************************************************
// TREE-method: forces all children to load
Procedure TGUIItem.Load;
Var LocalChild: TGUIItem; I: Integer;
Begin
   LoadOwnData;

   With FChildren Do If Count > 0
      Then For I:=0 To Count-1 Do Begin
         LocalChild:= List[I];
         LocalChild.Load;
      End;
End;
//********************************************************************
// Virtual method - descendents should overwrite
Procedure TGUIItem.LoadOwnData;
Var GUIPositionRect: TGUIPositionRect;
Begin
   With FDataProvider Do Begin
      FName:= LoadIdentity;
      FVisible:= LoadVisible;
      BGColorName:= LoadColorName;
      FGColorName:= LoadColorName;
   End;

   GUIPositionRect:= FDataProvider.LoadPositionRect;
   With GUIPositionRect Do Begin
      FLeft:= Left;
      FRight:= Right;
      FTop:= Top;
      FBottom:= Bottom;
   End;
End;
//********************************************************************
// TREE-method: forces all children to save
Procedure TGUIItem.Save;
Var LocalChild: TGUIItem; I: Integer;
Begin
   SaveOwnData;

   With FChildren Do If Count > 0
      Then For I:=0 To Count-1 Do Begin
         LocalChild:= List[I];
         LocalChild.Save;
      End;
End;
//********************************************************************
// Virtual method - descendents should overwrite
Procedure TGUIItem.SaveOwnData;
Var GUIPositionRect: TGUIPositionRect;
Begin
   With FDataProvider Do Begin
      SaveIdentity(FName);
      SaveVisible(FVisible);
      SaveColorName(FBGColorName);
      SaveColorName(FFGColorName);
   End;

   With GUIPositionRect Do Begin
      Left:= FLeft;
      Right:= FRight;
      Top:= FTop;
      Bottom:= FBottom;
   End;
   FDataProvider.SavePositionRect(GUIPositionRect);
End;
//********************************************************************
Procedure TGUIItem.SetBGColorName(AColorName: TGUIColorName);
Var LIndex: TGUIThemesIndex;
Begin
   LIndex:= Themes.ColorIndex(AColorName);
   If LIndex >= 0 Then Begin
      FBGColorIndex:= LIndex;
      FBGColorName:= AColorName;
   End;
End;
//********************************************************************
Procedure TGUIItem.SetFGColorName(AColorName: TGUIColorName);
Var LIndex: TGUIThemesIndex;
Begin
   LIndex:= Themes.ColorIndex(AColorName);
   If LIndex >= 0 Then Begin
      FFGColorIndex:= LIndex;
      FFGColorName:= AColorName;
   End;
End;
//********************************************************************
//   T G U I M A N A G E R
//********************************************************************
// No comment
Constructor TGUIManager.Create(AName: String; AParent: TGUIItem);
Begin
   Inherited Create(AName, AParent);

   FDataProvider:= Nil;
   FThemes:= TThemes.Create;
   GRS_Environment:= TGRS_Environment.Create;

   FBGColorIndex:= FThemes.ColorIndex('SolidBlue');
   FFGColorIndex:= FThemes.ColorIndex('SolidWhite');

   FGUIQuit:= False;
End;
//********************************************************************
// No comment
Destructor TGUIManager.Destroy;
Begin
   GRS_Environment.Free;
   FThemes.Free;
   FDataProvider.Free;

   Inherited Destroy;
End;
//********************************************************************
Procedure TGUIManager.CatchEvent;
Var GRS_EventType: TGRS_EventType;
    LKeyEventData: TKeyEventData;
    LMouseBtnEventData: TMouseBtnEventData;
Begin
   With GRS_Environment Do Begin
      GRS_Environment.CatchEvent;
      GRS_EventType:= GetEventType;
      Case GRS_EventType Of
         //-----------------------------------------------------------
         Quit: FGUIQuit:= True;
         //-----------------------------------------------------------
         Resize:
            Begin
               FLeft:=0; FTop:=0;
               FRight:= GetWindowWidth;
               FBottom:= GetWindowHeight;
            End;
         //-----------------------------------------------------------
         KeyDown:
            With FNoticeBoard Do If FocusedItem <> Nil Then Begin
               LKeyEventData:= GetKeyEventData;
               FocusedItem.OnKeyDown(LKeyEventData);
            End;
         //-----------------------------------------------------------
         KeyUp:
            With FNoticeBoard Do If FocusedItem <> Nil Then Begin
               LKeyEventData:= GetKeyEventData;
               FocusedItem.OnKeyUp(LKeyEventData);
            End;
         //-----------------------------------------------------------
         MouseBtnDown: Begin
            LMouseBtnEventData:= GetMouseBtnEventData;

            // GET FOCUS ON MOUSECLICK **********
            If LMouseBtnEventData.MouseBtn = MOUSEBTN_L
               Then GetFocusedItemOnMouseClick(LMouseBtnEventData);

            With FNoticeBoard Do If FocusedItem <> Nil Then Begin
               If LMouseBtnEventData.MouseBtn = MOUSEBTN_L
                  Then FocusedItem.OnMouseBtnLDown(LMouseBtnEventData);

               If LMouseBtnEventData.MouseBtn = MOUSEBTN_R
                  Then FocusedItem.OnMouseBtnRDown(LMouseBtnEventData);

               If LMouseBtnEventData.MouseBtn = MOUSEBTN_M
                  Then FocusedItem.OnMouseBtnMDown(LMouseBtnEventData);
            End;
         End;
         //-----------------------------------------------------------
         MouseBtnUp:
            With FNoticeBoard Do If FocusedItem <> Nil Then Begin
               LMouseBtnEventData:= GetMouseBtnEventData;

               If LMouseBtnEventData.MouseBtn = MOUSEBTN_L
                     Then FocusedItem.OnMouseBtnLUp(LMouseBtnEventData);
            End;
         //-----------------------------------------------------------
         // etc.
      End;
   End;
End;
//********************************************************************
Procedure TGUIManager.GetFocusedItemOnMouseClick
      (AMouseBtnEventData: TMouseBtnEventData);
Begin
   With FNoticeBoard Do Begin
      If FocusedItem <> Nil Then FocusedItem.Focus:= False;
      FocusedItem:= Nil;

      With AMouseBtnEventData Do Hit2D(FNoticeBoard,X,Y);

      If FocusedItem <> Nil Then Begin
         FNoticeBoard.FocusedItem.Focus:= True;
      End;
   End;
End;
//********************************************************************
Procedure TGUIManager.DrawGUI;
Begin
   GRS_Environment.BeforeDraw;
   Inherited Draw; // Draws own data and GUI-tree
   GRS_Environment.AfterDraw;
End;
//********************************************************************
Function TGUIManager.InitGraphicSupport: Boolean;
Var GRS_FatalError: TGRS_FatalError;
Begin
   GRS_FatalError:= GRS_Environment.Initialize(smWindow,640,480);
   With GRS_FatalError Do
      If (GRS_Video) And (GRS_Font) And (GRS_Library)
         Then Begin
            Result:= True;
            With GRS_Environment Do Begin
               FLeft:=0; FRight:= GetWindowWidth;
               FTop:=0; FBottom:= GetWindowHeight;
            End;
         End
         Else Result:= False;
End;
//********************************************************************
Procedure TGUIManager.SetDataProvider(ADataProvider:TGUIDataProvider);
Var LocalChild: TGUIItem; I: Integer;
Begin
   FDataprovider:= ADataProvider;
   Themes.DataProvider:= ADataProvider;

   With FChildren Do If Count > 0
      Then For I:=0 To Count-1 Do Begin
         LocalChild:= List[I];
         LocalChild.DataProvider:= ADataProvider;
      End;
End;
//********************************************************************
//********************************************************************
End.

