//********************************************************************
UNIT GraphicSupport;  
//********************************************************************
INTERFACE

USES SysUtils, Math,
     Sdl, Sdl_Image, Sdl_TTF, SdlUtils, dglOpenGL;
//********************************************************************
TYPE
   TPoint2D = Record X,Y: Integer; End;
   TRGBAColor = Record Red,Green,Blue,Alpha: Single; End;
   TGRS_ShowMode = (smWindow, smFullScreen);
//********************************************************************
CONST
     SolidWhite: TRGBAColor = (Red:1; Green:1; Blue:1; Alpha:1);
     SolidBlack: TRGBAColor = (Red:0; Green:0; Blue:0; Alpha:1);
      SolidGrey: TRGBAColor = (Red:0.5; Green:0.5; Blue:0.5; Alpha:1);
       SolidRed: TRGBAColor = (Red:1; Green:0; Blue:0; Alpha:1);
     SolidGreen: TRGBAColor = (Red:0; Green:1; Blue:0; Alpha:1);
      SolidBlue: TRGBAColor = (Red:0; Green:0; Blue:1; Alpha:1);

    SolidYellow: TRGBAColor = (Red:1; Green:1; Blue:0; Alpha:1);
      SolidCyan: TRGBAColor = (Red:0; Green:1; Blue:1; Alpha:1);
   SolidMagenta: TRGBAColor = (Red:1; Green:0; Blue:1; Alpha:1);
    SolidOrange: TRGBAColor = (Red:0.96; Green:0.32; Blue:0; Alpha:1);

//********************************************************************
   MOUSEBTN_L = 0;
   MOUSEBTN_R = 1;
   MOUSEBTN_M = 2;
//********************************************************************
TYPE
//********************************************************************
   TGRS_FatalError = Record
      GRS_Video, GRS_Library, GRS_Font: Boolean;
   End;
   TGRS_EventType = (None,Quit,Resize, KeyDown,KeyUp,
                     MouseBtnDown,MouseBtnUp,MouseMove,
                     MouseWheelDown,MouseWheelUp);

   TAdditionalKeys = Record  // Can be more than one
      ssNone,ssShift,ssAlt,ssCtrl: Boolean;
   End;
   TGRS_MouseBtnType = Byte; // Only one is possible
   TKeyEventData = Record
      Key: Word;
      AdditionalKeys: TAdditionalKeys;
   End;
   TMouseBtnEventData = Record
      MouseBtn: TGRS_MouseBtnType;
      X,Y: Integer
   End;
   TMouseMoveEventData = Record
      AdditionalKeys: TAdditionalKeys;
      X,Y: Integer
   End;
   TMouseWheelEventData = Record
      AdditionalKeys: TAdditionalKeys;
      X,Y: Integer
   End;
//********************************************************************
   IGRS_Environment = Interface
       Function Initialize(AGRS_ShowMode: TGRS_ShowMode;
            AWindowWidth,AWindowHeight: Integer): TGRS_FatalError;
      Procedure Finalize;

      Procedure BeforeDraw;
      Procedure AfterDraw;

      Procedure CatchEvent;
       Function GetEventType: TGRS_EventType;
       Function GetKeyEventData: TKeyEventData;
       Function GetMouseBtnEventData: TMouseBtnEventData;
       Function GetMouseMoveEventData: TMouseMoveEventData;
       Function GetMouseWheelEventData: TMouseWheelEventData;

       Function GetWindowHeight: Integer;
       Function GetWindowWidth: Integer;
   End;

   TGRS_Environment = Class(TInterfacedObject,IGRS_Environment)
   Private
      GRS_BitsPerPixel: TGlInt;
      SDLSurface: PSDL_Surface;
      GRS_ShowMode: TGRS_ShowMode;
      SDLVideoFlags: UInt32;
      SDLVideoInfo: PSDL_VideoInfo;
      FWindowHeight,FWindowWidth: Integer;

      FEventType: TGRS_EventType;
      FKeyEventData: TKeyEventData;
      FMouseBtnEventData: TMouseBtnEventData;
      FMouseMoveEventData: TMouseMoveEventData;
      FMouseWheelEventData: TMouseWheelEventData;

      Procedure InitDrawing;
   Public
      Function Initialize(AGRS_ShowMode: TGRS_ShowMode;
               AWindowWidth,AWindowHeight: Integer): TGRS_FatalError;
      Procedure Finalize;

      Procedure BeforeDraw;
      Procedure AfterDraw;

      Procedure CatchEvent;
       Function GetEventType: TGRS_EventType;
       Function GetKeyEventData: TKeyEventData;
       Function GetMouseBtnEventData: TMouseBtnEventData;
       Function GetMouseMoveEventData: TMouseMoveEventData;
       Function GetMouseWheelEventData: TMouseWheelEventData;

       Function GetWindowHeight: Integer;
       Function GetWindowWidth: Integer;
      Procedure WindowResize(AWidth,AHeight: Integer);
   End;
//********************************************************************
   IGRS_Font = Interface
       Function LoadFromFile: Boolean;
       Function BuildFont: Boolean;
      Procedure TextOut(XPos,YPos: Integer; AString: String);
      Procedure TextOutSelect(XPos,YPos,SelStart,SelEnd: Integer;
                     AString: String);
      Procedure SetColor(AColor: TRGBAColor);
      Procedure SetSize(APercentage: Integer);
      Procedure SetStyle(AStyle: Integer);
       Function TextHeight(AString: String): Integer;
       Function TextWidth(AString: String): Integer;
   End;

   TGRS_TTFont = Class(TInterfacedObject,IGRS_Font)
   Private
      FTTFont: PTTF_Font;
      FColor,FColorInverse: TRGBAColor;
      FLetter: GlUInt;
      FLetterHeightMax,FLetterWidthMax: Byte;
      FLetterWidth: Array[0..255] of Byte;
      FName: String;
      FSize: Integer;
      FStyle: Integer;
      FTexture: GluInt;
      FTextureHeight,FTextureWidth: Integer;
       Procedure BuildLetters;
        Function BmpCol(ASCIICode: Byte): Integer;
        Function BmpRow(ASCIICode: Byte): Integer;
        Function BmpSpace(ASCIICode: Byte): Integer;
        Function CalcTextureSize: Boolean;
        Function GenerateTexture: Boolean;
        Function InternalWidth(AParameter: Single): Single;
        Function LoadFromFile: Boolean;
   Public
      Constructor Create(AName: TFileName); Reintroduce;
      Destructor Destroy; Override;
        Function BuildFont: Boolean;
        Function GetName: String;
       Procedure TextOut(XPos,YPos: Integer; AString: String);
       Procedure TextOutSelect(XPos,YPos,SelStart,SelEnd: Integer;
                     AString: String);
       Procedure SetColor(AColor: TRGBAColor);
       Procedure SetSize(APercentage: Integer);
       Procedure SetStyle(AStyle: Integer);
        Function TextHeight(AString: String): Integer;
        Function TextWidth(AString: String): Integer;
   End;
//********************************************************************
   IGRS_Texture = Interface
       Function LoadFromFile(AName: String;
                     CreateMipMaps: Boolean): Boolean;
      Procedure Reload;
   End;

   TGRS_Texture = Class(TInterfacedObject,IGRS_Texture)
   Private
      FAlphaColor: TRGBAColor;
      FAlphaOnColor: Boolean;
      FMipMaps: Boolean;
      FName: String;
      FTexture: GluInt;
       Procedure TransformToRGBA(ASource: PSDL_Surface;
                  Var ARGBADestination: PSDL_Surface);
       Procedure GenTexture(ASurface: PSDL_Surface);
   Public
      Destructor Destroy; Override;
       Procedure FlipSurfaceHoriz(ASurface: PSDL_Surface);
        Function GetName: String;
        Function GetTexture: Gluint;
        Function LoadFromFile(AName: String;
                       CreateMipMaps: Boolean): Boolean;
       Procedure Reload;
       Procedure SetAlphaColor(AColor: TRGBAColor);
   End;
//********************************************************************
// D R A W I N G   P R O C E D U R E S
//********************************************************************
Procedure GRS_Ellipse2D(ALeft,ATop,ARight,ABottom: Integer;
                             Fill: Boolean);
Procedure GRS_Line2D(FromX,FromY,ToX,ToY: Integer);
Procedure GRS_Graph2D(APoint: Array Of TPoint2D);
Procedure GRS_Polygon2D(APoint: Array Of TPoint2D; Fill: Boolean);
Procedure GRS_Rectangle2D(ALeft,ATop,ARight,ABottom: Integer;
                             Fill: Boolean);
Procedure GRS_RoundRect2D(ALeft,ATop,ARight,ABottom: Integer;
                             Fill: Boolean);
Procedure GRS_SetColor(AColor: TRGBAColor);
Procedure GRS_SetLineWidth(ALineWidth: Integer);
//********************************************************************
IMPLEMENTATION
//********************************************************************
CONST  PiDiv180 = 3.14159/180;

VAR    MaskR,MaskG,MaskB,MaskA: Uint32;
//********************************************************************
Procedure SetupByteOrder;
Begin
   If SDL_BYTEORDER = SDL_BIG_ENDIAN
      Then Begin
         MaskR:=$ff000000;
         MaskG:=$00ff0000;
         MaskB:=$0000ff00;
         MaskA:=$000000ff;
      End
      Else Begin
         MaskR:=$000000ff;
         MaskG:=$0000ff00;
         MaskB:=$00ff0000;
         MaskA:=$ff000000;
      End;
End;
//********************************************************************
//********************************************************************
// T G R S _ E N V I R O N M E N T
//********************************************************************
Function TGRS_Environment.Initialize(AGRS_ShowMode: TGRS_ShowMode;
            AWindowWidth,AWindowHeight: Integer): TGRS_FatalError;

Begin
   With Result Do Begin
      GRS_Video:= True;
      GRS_Library:= True;
      GRS_Font:= True;
   End;

   // Initialize SDL -------------------------------------------------
   If (SDL_Init(SDL_INIT_VIDEO) < 0) Then Result.GRS_Video:= False;

   // Fetch Video Info -----------------------------------------------
   If Result.GRS_Video Then Begin
      SDLVideoInfo:= SDL_GetVideoInfo;
      If (SDLVideoInfo = Nil) Then Result.GRS_Video:= False;
   End;

   // Video Settings -------------------------------------------------
   If Result.GRS_Video Then Begin
      // Flags to pass to SDL_SetVideoMode ---------------------------
      SDLVideoFlags:= SDL_OPENGL;
      SDLVideoFlags:= SDLVideoFlags Or SDL_DOUBLEBUF;
      SDLVideoFlags:= SDLVideoFlags Or SDL_HWPALETTE;

      // Check if surfaces can be stored in memory -------------------
      If (SDLVideoInfo.hw_available <> 0)
         Then SDLVideoFlags:= SDLVideoFlags Or SDL_HWSURFACE
         Else SDLVideoFlags:= SDLVideoFlags Or SDL_SWSURFACE;

      // Checks if hardware blits can be done ------------------------
      If (SDLVideoInfo.Blit_hw <> 0)
         Then SDLVideoFlags:= SDLVideoFlags Or SDL_HWACCEL;

      // Set FullScreen / Window -------------------------------------
      // (2.Parameter = Icon)
      Case AGRS_ShowMode Of
             smWindow: Begin
                SDLVideoFlags:= SDLVideoFlags Or SDL_RESIZABLE;
                SDL_WM_SetCaption('', Nil);
             End;
         smFullScreen: SDLVideoFlags:= SDLVideoFlags Or SDL_FULLSCREEN;
      End;

      // Set OpenGL Attributes (Pixel Format) ------------------------
      SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 5);
      SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 5);
      SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 5);
      SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 16);
      SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
      //SDL_GL_SetAttribute(SDL_GL_STENCIL_SIZE, 8);

      // Set Video Mode ----------------------------------------------
      Case AGRS_ShowMode Of
             smWindow: Begin
                          FWindowWidth:= AWindowWidth;
                          FWindowHeight:= AWindowHeight;
                          GRS_BitsPerPixel:= 32;
                       End;
         smFullScreen: Begin
                          FWindowWidth:= 1024;
                          FWindowHeight:= 768;
                          GRS_BitsPerPixel:= 32;
                       End;
      End; // Of Case

      SDLSurface:= SDL_SetVideoMode(FWindowWidth,FWindowHeight,
                            GRS_BitsPerPixel,SDLVideoFlags);

      If SDLSurface = Nil Then Result.GRS_Video:= False;
   End;

   // Initialize True Type Font --------------------------------------
   If Result.GRS_Video Then Begin
      If (TTF_Init < 0) Then Result.GRS_Font:= False;
   End;

   // Initialize OPENGL ----------------------------------------------
   With Result Do If (GRS_Video) And (GRS_Font) Then Begin
      If Not InitOpenGl
         Then GRS_Library:= False
         Else Begin
            WindowResize(AWindowWidth,AWindowHeight);
            GRS_ShowMode:= AGRS_ShowMode;
            InitDrawing;
         End;
      End;
End;
//********************************************************************
Procedure TGRS_Environment.InitDrawing;
Begin
   glShadeModel(GL_SMOOTH);					 // Enable smooth shading
   glClearDepth(1.0);								 // Setup depth buffer
   glEnable(GL_DEPTH_TEST);					 // Enable depth testing
   glDepthFunc(GL_LEQUAL);					 // Setup depth testing
   //glEnable(GL_CULL_FACE);           // Enable Face-Culling

   //glCullFace(GL_BACK);              // Cull backfaces
   glEnable(GL_TEXTURE_2D);          // Activate 2D-texture
                                     // Perspective adjustment
   glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
End;
//********************************************************************
Procedure TGRS_Environment.Finalize;
Begin
   TTF_QUIT;
   SDL_QUIT;
End;
//********************************************************************
Procedure TGRS_Environment.WindowResize(AWidth,AHeight: Integer);
Begin
   FWindowWidth:= AWidth; FWindowHeight:= AHeight;
   glViewport(0,0,FWindowWidth,FWindowHeight);
End;
//********************************************************************
Procedure TGRS_Environment.BeforeDraw;
Begin
   glClear(GL_COLOR_BUFFER_BIT Or GL_DEPTH_BUFFER_BIT);

   glPushMatrix;
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity;
   glOrtho(0,FWindowWidth,FWindowHeight,0,+1,-1);

   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity;
End;
//********************************************************************
Procedure TGRS_Environment.AfterDraw;
Begin
   glPopMatrix;
   SDL_GL_SwapBuffers;
End;
//********************************************************************
// This is a "translator" of events
Procedure TGRS_Environment.CatchEvent;
Var SDL_Event: TSDL_Event;
Begin
   FEventType:= None;

   If SDL_PollEvent(@SDL_Event) = 1
      Then Case SDL_Event.Type_ Of
      // QUITEVENT ---------------------------------------------------
         SDL_QUITEV: FEventType:= Quit;
      // RESIZEEVENT -------------------------------------------------
         SDL_VIDEORESIZE: Begin
            FEventType:= Resize;
            With SDL_Event.Resize Do WindowResize(W,H);
         End;
      // KEYEVENT ----------------------------------------------------
         SDL_KEYDOWN,SDL_KEYUP: Begin

         If SDL_Event.Type_ = SDL_KEYDOWN
            Then FEventType:= KeyDown
            Else FEventType:= KeyUp;

            With FKeyEventData Do Begin
               Key:= SDL_Event.Key.Keysym.Sym;  //&&&&&

               With AdditionalKeys,SDL_Event.Key.Keysym Do Begin
                  If (Modifier And KMOD_LSHIFT) = KMOD_LSHIFT
                     Then Begin ssNone:= False; ssShift:= True; End
                     Else Begin ssNone:= True; ssShift:= False; End;
                  If (Modifier And KMOD_LALT) = KMOD_LALT
                     Then Begin ssNone:= False; ssAlt:= True; End
                     Else Begin ssNone:= True; ssAlt:= False; End;
                  If (Modifier And KMOD_LCTRL) = KMOD_LCTRL
                     Then Begin ssNone:= False; ssCtrl:= True; End
                     Else Begin ssNone:= True; ssCtrl:= False; End;
               End;
            End;
         End;
      // MOUSEBUTTONEVENT --------------------------------------------
         SDL_MOUSEBUTTONDOWN,SDL_MOUSEBUTTONUP: Begin
            If SDL_Event.Type_ = SDL_MOUSEBUTTONDOWN
               Then FEventType:= MouseBtnDown
               Else FEventType:= MouseBtnUp;
            With FMouseBtnEventData Do Begin
               Case SDL_Event.Button.Button Of
                  SDL_BUTTON_LEFT: MouseBtn:= MOUSEBTN_L;
                  SDL_BUTTON_RIGHT: MouseBtn:= MOUSEBTN_R;
                  SDL_BUTTON_MIDDLE: MouseBtn:= MOUSEBTN_M;
               End;
               X:= SDL_Event.Button.X;
               Y:= SDL_Event.Button.Y;
            End;
            With FMouseWheelEventData Do Begin
               Case SDL_Event.Button.Button Of
                  SDL_BUTTON_WHEELDOWN: FEventType:= MouseWheelDown;
                  SDL_BUTTON_WHEELUP: FEventType:= MouseWheelUp;
               End;
               With AdditionalKeys,SDL_Event.Key.Keysym Do Begin
                  If (Modifier And KMOD_LSHIFT) = KMOD_LSHIFT
                     Then Begin ssNone:= False; ssShift:= True; End
                     Else Begin ssNone:= True; ssShift:= False; End;
                  If (Modifier And KMOD_LALT) = KMOD_LALT
                     Then Begin ssNone:= False; ssAlt:= True; End
                     Else Begin ssNone:= True; ssAlt:= False; End;
                  If (Modifier And KMOD_LCTRL) = KMOD_LCTRL
                     Then Begin ssNone:= False; ssCtrl:= True; End
                     Else Begin ssNone:= True; ssCtrl:= False; End;
               End;
               X:= SDL_Event.Button.X;
               Y:= SDL_Event.Button.Y;
            End;
         End;
      // MOUSEMOTIONEVENT --------------------------------------------
         SDL_MOUSEMOTION: Begin
            FEventType:= MouseMove;
            With FMouseMoveEventData Do Begin
               With AdditionalKeys,SDL_Event.Key.Keysym Do Begin
                  If (Modifier And KMOD_LSHIFT) = KMOD_LSHIFT
                     Then Begin ssNone:= False; ssShift:= True; End
                     Else Begin ssNone:= True; ssShift:= False; End;
                  If (Modifier And KMOD_LALT) = KMOD_LALT
                     Then Begin ssNone:= False; ssAlt:= True; End
                     Else Begin ssNone:= True; ssAlt:= False; End;
                  If (Modifier And KMOD_LCTRL) = KMOD_LCTRL
                     Then Begin ssNone:= False; ssCtrl:= True; End
                     Else Begin ssNone:= True; ssCtrl:= False; End;
               End;
               X:= SDL_Event.Button.X;
               Y:= SDL_Event.Button.Y;
            End;
         End;
      //--------------------------------------------------------------
      End;
End;
//********************************************************************
Function TGRS_Environment.GetEventType: TGRS_EventType;
Begin
   Result:= FEventType;
End;
//********************************************************************
Function TGRS_Environment.GetKeyEventData: TKeyEventData;
Begin
   Result:= FKeyEventData;
End;
//********************************************************************
Function TGRS_Environment.GetMouseBtnEventData: TMouseBtnEventData;
Begin
   Result:= FMouseBtnEventData;
End;
//********************************************************************
Function TGRS_Environment.GetMouseMoveEventData: TMouseMoveEventData;
Begin
   Result:= FMouseMoveEventData;
End;
//********************************************************************
Function TGRS_Environment.GetMouseWheelEventData: TMouseWheelEventData;
Begin
   Result:= FMouseWheelEventData;
End;
//********************************************************************
Function TGRS_Environment.GetWindowHeight: Integer;
Begin
   Result:= FWindowHeight;
End;
//********************************************************************
Function TGRS_Environment.GetWindowWidth: Integer;
Begin
   Result:= FWindowWidth;
End;
//********************************************************************
//********************************************************************
//   T G R S _ F O N T
//********************************************************************
// No comment
Constructor TGRS_TTFont.Create(AName: TFileName);
Begin
   Inherited Create;
   FTTFont:= Nil;
   SetSize(100);
   SetStyle(TTF_STYLE_NORMAL);
End;
//********************************************************************
// No comment
Destructor TGRS_TTFont.Destroy;
Begin
   glDeleteLists(FLetter,256);
   //TTF_CloseFont(FTTFont); // Is done by TTF_QUIT
   Inherited Destroy;
End;
//********************************************************************
// No comment
Function TGRS_TTFont.BmpCol(ASCIICode: Byte): Integer;
Begin Result:= (ASCIICode Mod 16); End;
//********************************************************************
// No comment
Function TGRS_TTFont.BmpRow(ASCIICode: Byte): Integer;
Begin Result:= (ASCIICode Div 16) + 1; End;
//********************************************************************
// No comment
Function TGRS_TTFont.BmpSpace(ASCIICode: Byte): Integer;
Begin Result:= (FLetterWidthMax-FLetterWidth[ASCIICode]) Div 2; End;
//********************************************************************
// No comment
Function TGRS_TTFont.BuildFont: Boolean;
Begin
   Result:= False;

   If LoadFromFile Then Begin
      TTF_SetFontStyle(FTTFont,FStyle);
      If (CalcTextureSize) And (GenerateTexture) Then Begin
         BuildLetters;
         Result:= True;
      End;
   End;
End;
//********************************************************************
// Create 256 display-lists, one for each letter
Procedure TGRS_TTFont.BuildLetters;
Var TxLeft,TxRight,TxBottom,TxTop,
    VxLeft,VxRight,VxBottom,VxTop: Single;
    I: Integer;
Begin
   If FLetter > 0 Then glDeleteLists(FLetter,256);
   FLetter:= glGenLists(256);

   For I:= 0 To 255 Do Begin
      TxLeft:= (BmpCol(I)*FLetterWidthMax + BmpSpace(I))
                            /FTextureWidth;
      TxRight:= TxLeft + FLetterWidth[I]/FTextureWidth;
      TxBottom:= (FTextureHeight-BmpRow(I)*FLetterHeightMax)
                            /FTextureHeight;
      TxTop:= TxBottom + FLetterHeightMax/FTextureHeight;

      VxLeft:= 0;
      VxRight:= InternalWidth(FLetterWidth[I]);
      VxBottom:= 0;
      VxTop:= InternalWidth(FLetterHeightMax);

      glNewList(FLetter+I, GL_COMPILE);
      glBegin(GL_QUADS);
         glTexCoord2f(TxLeft, TxBottom); glVertex2f(VxLeft, VxBottom);
         glTexCoord2f(TxRight,TxBottom); glVertex2f(VxRight,VxBottom);
         glTexCoord2f(TxRight,TxTop);    glVertex2f(VxRight,VxTop);
         glTexCoord2f(TxLeft, TxTop);    glVertex2f(VxLeft, VxTop);
      glEnd();
      glTranslatef(InternalWidth(FLetterWidth[I]),0,0);
      glEndList();
   End;
End;
//********************************************************************
// No comment
Function TGRS_TTFont.CalcTextureSize: Boolean;
Var MinX,MaxX,MinY,MaxY,Advance,I: Integer;
Begin
   // Read letterwidths ----------------------------------------------
   FLetterWidthMax:= 0;
   For I:= 0 to 255 do
   Begin
     TTF_GlyphMetrics(FTTFont,I,MinX,MaxX,MinY,MaxY,Advance);
     FLetterWidth[I]:= Advance;
     If Max(Advance,MaxX) > FLetterWidthMax
        Then FLetterWidthMax:= Max(Advance,MaxX);
   End;

   // Calculate size of texture --------------------------------------
   FLetterHeightMax:= TTF_FontHeight(FTTFont);
   I:=0;
   Repeat Inc(I);
   Until (FLetterWidthMax*16) <= Power(2,I);
   FTextureWidth:= Trunc(Power(2,I));
   I:=0;
   Repeat Inc(I);
   Until (FLetterHeightMax*16) <= Power(2,I);
   FTextureHeight:= Trunc(Power(2,I));

   // Check if texture size too big ----------------------------------
   glGetIntegerv(GL_MAX_TEXTURE_SIZE,@I);
   If Max(FTextureWidth,FTextureHeight) <= I
      Then Result:= True
      Else Result:= False;
End;
//********************************************************************
// Creates an OpenGl texture from a SDL surface
Function TGRS_TTFont.GenerateTexture: Boolean;
Const
   SDLSolidWhite: TSDL_Color = (R:1; G:1; B:1; Unused:1);
Var
   Area: TSDL_Rect;
   FontSurface: PSDL_Surface;
   ImageOK: Boolean;
   Letter: String;
   LetterSurface: PSDL_Surface;
   //GRS_Texture: TGRS_Texture;
   I: Integer;
Begin
   // Create FontSurface for font-texture
   FontSurface:= SDL_CreateRGBSurface(SDL_SWSURFACE,
      FTextureWidth,FTextureHeight,32,MaskR,MaskG,MaskB,MaskA);

   If (FontSurface <> Nil) Then ImageOK:= True
                           Else ImageOK:= False;
   If ImageOK
   Then Begin
      //GRS_Texture:= TGRS_Texture.Create;

      // Print letters and copy them to the font-texture
      For I:= 0 To 255 Do Begin
         Letter:= Chr(I);
         LetterSurface:= TTF_RenderText_Solid
                   (FTTFont,PChar(Letter),SDLSolidWhite);
         If LetterSurface <> Nil Then Begin
            //GRS_Texture.FlipSurfaceHoriz(LetterSurface);
            With Area Do Begin
               X:= BmpCol(I)*FLetterWidthMax + BmpSpace(I);
               Y:= FTextureHeight-BmpRow(I)*FLetterHeightMax;
               W:= LetterSurface.W;
               H:= LetterSurface.H;
            End;
            SDL_BlitSurface(LetterSurface,Nil,FontSurface,@Area);
            SDL_FreeSurface(LetterSurface);
            LetterSurface:= Nil;
         End;
      End;
      //GRS_Texture.Free;

      glGenTextures(1, @FTexture);
      glBindTexture(GL_TEXTURE_2D, FTexture);
      glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
      glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
      With FontSurface^ Do glTexImage2D(GL_TEXTURE_2D,
         0,GL_RGBA,W,H,0,GL_RGBA,GL_UNSIGNED_BYTE,Pixels);

      SDL_FreeSurface(FontSurface);

      Result:= True;
   End
   Else Result:= False;
End;
//********************************************************************
// No comment
Function TGRS_TTFont.GetName: String;
Begin Result:= FName; End;
//********************************************************************
// No comment
Function TGRS_TTFont.InternalWidth(AParameter: Single): Single;
Begin Result:= Trunc(AParameter*FSize/100); End;
//********************************************************************
// No comment
Function TGRS_TTFont.LoadFromFile: Boolean;
Begin
   Result:= False;
   If FTTFont <> Nil Then Begin
      TTF_CloseFont(FTTFont); FTTFont:= Nil;
   End;

   // TextureHeightMax is calculated during building the font in the
   // method "CalcTextureSize". Therefore on rebuilding the font it
   // has to be reset to 16
   FLetterHeightMax:= 16;

   FTTFont:= TTF_OpenFont(PChar(FName + '.TTF'),
             Trunc(FLetterHeightMax*FSize/100));
	 If FTTFont <> Nil Then Result:= True;
End;
//********************************************************************
// String output
Procedure TGRS_TTFont.TextOut(XPos,YPos: Integer; AString: String);
Begin
   glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
   glEnable(GL_BLEND);
   glBindTexture(GL_TEXTURE_2D,FTexture);
   With FColor Do glColor4f(Red,Green,Blue,Alpha);

   glPushMatrix;
   glLoadIdentity;
   glTranslatef(XPos,YPos,0);
   glListBase(FLetter);
   glCallLists(Length(AString),GL_UNSIGNED_BYTE,PChar(AString));
   glPopMatrix;
End;
//********************************************************************
// String output with selection
Procedure TGRS_TTFont.TextOutSelect(XPos,YPos,SelStart,SelEnd: Integer;
                                      AString: String);
Var I,Len: Integer;
Begin
   glEnable(GL_BLEND);
   glBindTexture(GL_TEXTURE_2D,FTexture);

   glPushMatrix; //------------------------------------------------
   glLoadIdentity;
   glTranslatef(XPos,YPos,0);
   glListBase(FLetter);
   Len:= Length(AString);
   For I:= 0 To Len-1 Do Begin
      If (I < SelStart) Or (I > SelEnd)
         Then Begin // not selected
            glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
            With FColor Do glColor4f(Red,Green,Blue,Alpha);
         End
         Else Begin // selected
            glBlendFunc(GL_SRC_ALPHA, GL_ZERO);
            With FColorInverse Do
               glColor4f(Red,Green,Blue,Alpha);
         End;
      glCallList(FLetter+Ord(AString[I+1]));
   End;
   glPopMatrix; //-------------------------------------------------
End;
//********************************************************************
// No comment
Procedure TGRS_TTFont.SetColor(AColor: TRGBAColor);
Begin
   FColor:= AColor;
   With FColorInverse Do Begin
      Red:= 1 - FColor.Red;
      Green:= 1 - FColor.Green;
      Blue:= 1 - FColor.Blue;
      Alpha:= 1;
   End;
End;
//********************************************************************
// No comment
Procedure TGRS_TTFont.SetSize(APercentage: Integer);
Begin FSize:= APercentage; End;
//********************************************************************
// No comment
Procedure TGRS_TTFont.SetStyle(AStyle: Integer);
Begin FStyle:= AStyle; End;
//********************************************************************
// No comment
Function TGRS_TTFont.TextHeight(AString: String): Integer;
Begin Result:= Trunc(FLetterHeightMax); End;
//********************************************************************
// No comment
Function TGRS_TTFont.TextWidth(AString: String): Integer;
Var I,Len: Integer; Letter: Char;
Begin
   Result:= 0;
   Len:= Length(AString);
   For I:= 1 To Len Do Begin
      Letter:= AString[I];
      Result:= Result+Trunc(FLetterWidth[Ord(Letter)]);
   End;
End;
//********************************************************************
//********************************************************************
//   T G R S _ T E X T U R E
//********************************************************************
// No comment
Destructor TGRS_Texture.Destroy;
Begin
   If FTexture > 0
      Then glDeleteTextures(1, @FTexture);
   Inherited Destroy;
End;
//********************************************************************
// No comment
Procedure TGRS_Texture.TransformToRGBA(ASource: PSDL_Surface;
                    Var ARGBADestination: PSDL_Surface);
Var LRed, LGreen, LBlue, LAlpha: UInt8;
    PRed, PGreen, PBlue, PAlpha: PUInt8;
    Pixel: Uint32; X,Y: Integer;
Begin
   With ASource^ Do ARGBADestination:= SDL_CreateRGBSurface
         (SDL_SWSURFACE,W,H,32,MaskR,MaskG,MaskB,MaskA);

   PRed:=@LRed;
   PGreen:=@LGreen;
   PBlue:=@LBlue;
   PAlpha:=@LAlpha;

   For Y:= 0 To ASource.H-1 Do
      For X:= 0 To ASource.W-1 Do Begin
         Pixel:= SDL_GetPixel(ASource,X,Y);

         SDL_GetRGBA(Pixel,ASource.Format,PRed,PGreen,PBlue,PAlpha);

         If FAlphaOnColor
            Then Begin
               With FAlphaColor Do
                  If (LRed=Red) And (LGreen=Green) And (LBlue=Blue)
                     Then Alpha:=0
                     Else Alpha:=255;
            End
            Else LAlpha:=255;

         Pixel:= SDL_MapRGBA
            (ARGBADestination.Format,LRed,LGreen,LBlue,LAlpha);
         SDL_PutPixel(ARGBADestination,X,Y,Pixel);
      End;
End;
//********************************************************************
// Exchange YMin/YMax for the specific texture
Procedure TGRS_Texture.FlipSurfaceHoriz(ASurface: PSDL_Surface);
Var X,Y: Integer; A,B: UInt32;
Begin
   For Y:= 0 To ((ASurface.H-1) Div 2) Do
      For X:= 0 To ASurface.W-1 Do Begin
         A:= SDL_GetPixel(ASurface,X,Y);
         B:= SDL_GetPixel(ASurface,X,ASurface.H-1-Y);
         SDL_PutPixel(ASurface,X,Y,B);
         SDL_PutPixel(ASurface,X,ASurface.H-1-Y,A);
      End;
End;
//********************************************************************
// No comment
Procedure TGRS_Texture.GenTexture(ASurface: PSDL_Surface);
Var LCompression: Longint; LRGBFormat: Cardinal;
    MipMapsWithOpenGl: Boolean;
Begin
   If (ASurface.Format.BytesPerPixel = 4)
      Then Begin
         LCompression:= GL_COMPRESSED_RGBA_ARB;
         LRGBFormat:= GL_RGBA;
      End
      Else Begin
         LCompression:= GL_COMPRESSED_RGB_ARB;
         LRGBFormat:= GL_RGB;
      End;
   If (FMipMaps) And (GL_VERSION_1_4 Or GL_SGIS_generate_mipmap)
      Then MipMapsWithOpenGl:= True
      Else MipMapsWithOpenGl:= False;

   FlipSurfaceHoriz(ASurface);

   glGenTextures(1, @FTexture);
   glBindTexture(GL_TEXTURE_2D, FTexture);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

   If FMipMaps
      Then Begin
         glTexParameteri(GL_TEXTURE_2D,
                 GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
         glTexParameteri(GL_TEXTURE_2D,
                 GL_GENERATE_MIPMAP,GL_TRUE);

         If (MipMapsWithOpenGl)
            Then With ASurface^ Do glTexImage2D(GL_TEXTURE_2D,
                  0,LCompression,W,H,0,LRGBFormat,GL_UNSIGNED_BYTE,Pixels)
            Else With ASurface^ Do gluBuild2DMipmaps(GL_TEXTURE_2D,
                  LCompression,W,H,LRGBFormat,GL_UNSIGNED_BYTE,Pixels);
      End
      Else Begin
         glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
         With ASurface^ Do glTexImage2D(GL_TEXTURE_2D,
                  0,LCompression,W,H,0,LRGBFormat,GL_UNSIGNED_BYTE,Pixels)
      End;

   { // Check compression:
   glGetTexLevelParameteriv(GL_TEXTURE_2D,0,
          GL_TEXTURE_COMPRESSED_ARB, @compressed);
   If Compressed <> 1 Then SDLLogAdd('Could not compress texture'); }
End;
//********************************************************************
// No comment
Function TGRS_Texture.GetName: String;
Begin Result:= FName; End;
//********************************************************************
// No comment
Function TGRS_Texture.GetTexture: Gluint;
Begin Result:= FTexture; End;
//********************************************************************
// No comment
Function TGRS_Texture.LoadFromFile(AName: String;
                      CreateMipMaps: Boolean): Boolean;
Var LoadImage,RGBAImage: PSDL_Surface;
    LocalFileName: String; ImageLoad: Boolean;
Begin
   FMipMaps:= CreateMipMaps;

   LoadImage:= Nil;
   RGBAImage:= Nil;

   ImageLoad:= False;

   If Not ImageLoad Then Begin
      LocalFileName:= AName + '.JPG';
      If FileExists(LocalFileName) Then Begin
         LoadImage:= IMG_Load(PChar(LocalFileName));
         ImageLoad:= True;
      End;
   End;
   If Not ImageLoad Then Begin
      LocalFileName:= AName + '.PNG';
      If FileExists(LocalFileName) Then Begin
         LoadImage:= IMG_Load(PChar(LocalFileName));
         ImageLoad:= True;
      End;
   End;
   If Not ImageLoad Then Begin
      LocalFileName:= AName + '.TGA';
      If FileExists(LocalFileName) Then Begin
         LoadImage:= IMG_Load(PChar(LocalFileName));
         ImageLoad:= True;
      End;
   End;
   If Not ImageLoad Then Begin
      LocalFileName:= AName + '.BMP';
      If FileExists(LocalFileName) Then Begin
         LoadImage:= IMG_Load(PChar(LocalFileName));
         ImageLoad:= True;
      End;
   End;

   If (LoadImage <> Nil)
      Then Begin
         FName:= AName;
         TransformToRGBA(LoadImage,RGBAImage);
         GenTexture(RGBAImage);
         Result:= True;
      End
      Else Begin
         FTexture:= 0;
         Result:= False;
      End;

   If LoadImage <> Nil Then SDL_FreeSurface(LoadImage);
   If RGBAImage <> Nil Then SDL_FreeSurface(RGBAImage);
End;
//********************************************************************
// No comment
Procedure TGRS_Texture.Reload;
Begin LoadFromFile(FName,FMipMaps); End;
//********************************************************************
// No comment
Procedure TGRS_Texture.SetAlphaColor(AColor: TRGBAColor);
Begin
   FAlphaColor:= AColor;
   FAlphaOnColor:= True;
End;
//********************************************************************
//********************************************************************
// D R A W I N G   P R O C E D U R E S
//********************************************************************
// PRIVATE: Generates points for a piece of an ellipse
// (counterclockwise)
Procedure GRS_Arcus2D(ALeft,ATop,ARight,ABottom: Single;
                     BeginDegrees,EndDegrees: Integer);
Var DegInRad,CenterX,CenterY,X,Y: Single; I: Integer;
Begin
   X:= (ARight-ALeft)/2;
   CenterX:= ALeft + X;

   Y:= (ABottom-ATop)/2;
   CenterY:= ATop + Y;

   I:=BeginDegrees;
   While I < EndDegrees Do Begin
      DegInRad:= I*PiDiv180;
      glVertex2f(CenterX + Cos(DeginRad)* X,
                 //CenterY + Sin(DeginRad)* Y);  // 0/0 = Left/Bottom
                 CenterY - Sin(DeginRad)* Y);    // 0/0 = Left/Top
      Inc(I,5); //===============>
   End;
End;
//********************************************************************
// Draws a 2D-Ellipse
Procedure GRS_Ellipse2D(ALeft,ATop,ARight,ABottom: Integer;
                                  Fill: Boolean);
Begin
   If Fill Then glPolygonMode(GL_FRONT_AND_BACK,GL_FILL)
           Else glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);

   glEnable(GL_LINE_SMOOTH);

   glBegin(GL_POLYGON);
      GRS_Arcus2D(ALeft,ATop,ARight,ABottom,0,360);
   glEnd;
End;
//********************************************************************
// Draws a 2D-line from startpoint to endpoint
Procedure GRS_Line2D(FromX,FromY,ToX,ToY: Integer);
Begin
   //glLineStipple(1,$FFFF);
   glBegin(GL_LINES);
      glVertex2i(FromX,FromY);
      glVertex2i(ToX,ToY);
   glEnd;
End;
//********************************************************************
// Point plotter
Procedure GRS_Graph2D(APoint: Array Of TPoint2D);
Var I,Count: Integer;
Begin
   Count:= Length(APoint);
   glBegin(GL_POINTS);
      For I:= 0 To Count-1 Do With APoint[I] Do glVertex2i(X,Y);
   glEnd;
End;
//********************************************************************
// Draws a polygon either with lines only or filled with current color
Procedure GRS_Polygon2D(APoint: Array Of TPoint2D; Fill: Boolean);
Var I,Count: Integer;
Begin
   glEnable(GL_LINE_SMOOTH);
   If Fill Then glPolygonMode(GL_FRONT_AND_BACK,GL_FILL)
           Else glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
   Count:= Length(APoint);
   glBegin(GL_POLYGON);
      For I:= 0 To Count-1 Do With APoint[I] Do glVertex2f(X,Y);
   glEnd;
End;
//********************************************************************
Procedure GRS_Rectangle2D(ALeft,ATop,ARight,ABottom: Integer;
                             Fill: Boolean);
Begin
   If Fill Then glPolygonMode(GL_FRONT_AND_BACK,GL_FILL)
           Else glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);

   glEnable(GL_LINE_SMOOTH);

   glBegin(GL_QUADS);
      glVertex2i(ALeft,ABottom);
      glVertex2i(ARight,ABottom);
      glVertex2i(ARight,ATop);
      glVertex2i(ALeft,ATop);
   glEnd;
End;
//********************************************************************
Procedure GRS_RoundRect2D(ALeft,ATop,ARight,ABottom: Integer;
                             Fill: Boolean);
Const Factor = 0.3;
Var X,Y: Single;
Begin
   X:= (ARight-ALeft)*Factor;
   Y:= (ABottom-ATop)*Factor;

   If Fill Then glPolygonMode(GL_FRONT_AND_BACK,GL_FILL)
           Else glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);

   glEnable(GL_LINE_SMOOTH);

   glBegin(GL_POLYGON);
      // Lower left corner
      GRS_Arcus2D(ALeft,ABottom-Y,ALeft+X,ABottom,180,270);

      // Lower right corner
      GRS_Arcus2D(ARight-X,ABottom-Y,ARight,ABottom,270,360);

      // Upper right corner
      GRS_Arcus2D(ARight-X,ATop,ARight,ATop+Y,0,90);

      // Upper left corner
      GRS_Arcus2D(ALeft,ATop,ALeft+X,ATop+Y,90,180);
   glEnd;
End;
//********************************************************************
// No comment
Procedure GRS_SetColor(AColor: TRGBAColor);
Begin With AColor Do glColor4f(Red,Green,Blue,Alpha); End;
//********************************************************************
// No comment
Procedure GRS_SetLineWidth(ALineWidth: Integer);
Begin glLineWidth(ALineWidth); End;
//********************************************************************
//********************************************************************
End.
